# -*- coding: utf-8 -*-
from .ui.CID_load_shp_parcel import CIDLoadShpParcelDlg
from .ui.file_custom_widget import FileWidgetFactory, FileWidgetFormatter
from .composer.cid_inventory_generator import CIDPvGenerator
from PyQt5.QtCore import QUrl
from PyQt5.QtGui import QIcon, QDesktopServices
from PyQt5.QtWidgets import QAction
from qgis.core import QgsMessageLog, QgsVectorLayer, QgsProject, QgsGeometry,\
    QgsFeature, QgsApplication
from qgis.gui import QgsMapToolIdentifyFeature, QgsMapToolEmitPoint, QgsGui
from .db import get_connection
# Initialize Qt resources from file resources.py
from .resources import *
from .CID_layers import CIDLayers
import os.path
import tempfile


class CIDInventoryEncoding:
    """Encodes the CID inventory"""

    def __init__(self, iface):
        """Constructor"""
        self.iface = iface
        self.mapCanvas = iface.mapCanvas()
        self.load_shp_dlg = None
        self.menu = u'&CID'
        self.toolbar = self.iface.addToolBar(u'CID')
        self.toolbar.setObjectName(u'CID')
        self.layers = CIDLayers()

        self.maptool = None

        self.actions = dict()  # Dictionary where are stored the actions
        self.active_action_name = None  # The name of the action which is
        # active at the  moment
        self.stop_action_function = dict()  # Dictionary of function to
        # execute at the end of an action

        self.plugin_dir = os.path.dirname(__file__)

        self.file_widget_factory = FileWidgetFactory()
        self.file_widget_formatter = FileWidgetFormatter()
        QgsGui.editorWidgetRegistry().registerWidget("Base64 file", self.file_widget_factory)
        QgsApplication.fieldFormatterRegistry().addFieldFormatter(self.file_widget_formatter)

    def add_action(
        self, name, icon_path, text, callback, enabled_flag=True,
        add_to_menu=True, add_to_toolbar=True, status_tip=None,
        whats_this=None, parent=None, checkable=False
            ):
        """Add a toolbar icon to the toolbar.

        :param name: Name to get back the action from the dictionary of
            actions. The name will be the key of this dictionary.
        :type name: str

        :param icon_path: Path to the icon for this action. Can be a resource
            path (e.g. ':/plugins/foo/bar.png') or a normal file system path.
        :type icon_path: str

        :param text: Text that should be shown in menu items for this action.
        :type text: str

        :param callback: Function to be called when the action is triggered.
        :type callback: function

        :param enabled_flag: A flag indicating if the action should be enabled
            by default. Defaults to True.
        :type enabled_flag: bool

        :param add_to_menu: Flag indicating whether the action should also
            be added to the menu. Defaults to True.
        :type add_to_menu: bool

        :param add_to_toolbar: Flag indicating whether the action should also
            be added to the toolbar. Defaults to True.
        :type add_to_toolbar: bool

        :param status_tip: Optional text to show in a popup when mouse pointer
            hovers over the action.
        :type status_tip: str

        :param parent: Parent widget for the new action. Defaults None.
        :type parent: QWidget

        :param whats_this: Optional text to show in the status bar when the
            mouse pointer hovers over the action.

        :param checkable: Flag indicating if the action is checkable.
            Defaults to False.
        :type checkable: bool

        :returns: The action that was created. Note that the action is also
            added to self.actions dict.
        :rtype: QAction
        """

        icon = QIcon(icon_path)
        action = QAction(icon, text, parent)
        action.triggered.connect(callback)
        action.setEnabled(enabled_flag)

        if status_tip is not None:
            action.setStatusTip(status_tip)

        if whats_this is not None:
            action.setWhatsThis(whats_this)

        if add_to_toolbar:
            self.toolbar.addAction(action)

        if add_to_menu:
            self.iface.addPluginToMenu(
                self.menu,
                action)

        if checkable:
            action.setCheckable(True)

        self.actions[name] = action
        return action

    def initGui(self):
        """Create the menu entries and toolbar icons inside the QGIS GUI."""

        icon_path = ':/plugins/PAGGF/img/icon.png'
        self.add_action(
            'load_shp_dlg_display', icon_path,
            text="Import d'une parcelle après CID",
            callback=self.load_shp_dlg_display, parent=self.iface.mainWindow())

        self.add_action(
            'add_vertex', ':/plugins/PAGGF/img/icon_cid_vertex_add.png',
            text='Ajouter un sommet', callback=self.add_vertex,
            parent=self.iface.mainWindow(), checkable=True)
        self.stop_action_function['add_vertex'] = self.commit_changes

        self.add_action(
            'move_vertex', ':/plugins/PAGGF/img/icon_cid_vertex_move.png',
            text='Déplacer les sommets', callback=self.move_vertex,
            parent=self.iface.mainWindow(), checkable=True)
        self.stop_action_function['move_vertex'] = self.commit_changes

        self.add_action(
            'delete_vertex', ':/plugins/PAGGF/img/icon_cid_vertex_delete.png',
            text='Supprimer des sommets', callback=self.delete_vertex,
            parent=self.iface.mainWindow(), checkable=True)
        self.stop_action_function['delete_vertex'] = self.commit_changes

        self.add_action(
            'add_ring', ':/plugins/PAGGF/img/icon_cid_ring_add.png',
            text='Ajouter un anneau', callback=self.add_ring,
            parent=self.iface.mainWindow(), checkable=True)
        self.stop_action_function['add_ring'] = self.commit_changes

        self.add_action(
            'reshape', ':/plugins/PAGGF/img/icon_cid_reshape.png',
            text='Remodeler les entités', callback=self.reshape,
            parent=self.iface.mainWindow(), checkable=True)
        self.stop_action_function['reshape'] = self.commit_changes

        self.add_action(
            'edit_inventory',
            ':/plugins/PAGGF/img/icon_cid_edit_inventory.png',
            text='Éditer un inventaire', callback=self.edit_inventory,
            parent=self.iface.mainWindow(), checkable=True)
        self.stop_action_function['edit_inventory'] = self.commit_changes

        self.add_action(
            'print_inventory',
            ':/plugins/PAGGF/img/icon_cid_print_inventory.png',
            text='Imprimer un inventaire', callback=self.print_inventory,
            parent=self.iface.mainWindow(), checkable=True)
        self.stop_action_function['print_inventory'] = self.pan_trigger

        self.add_action(
            'validate_inventory',
            ':/plugins/PAGGF/img/icon_cid_validate_inventory.png',
            text='Valider un inventaire', callback=self.validate_inventory,
            parent=self.iface.mainWindow(), checkable=True)
        self.stop_action_function['validate_inventory'] = self.commit_changes

    def unload(self):
        """Removes the plugin menu item and icon from QGIS GUI."""
        for action in self.actions.values():
            self.iface.removePluginMenu('&CID', action)
            self.iface.removeToolBarIcon(action)
        # remove the toolbar
        del self.toolbar

    # QUESTION met on du code dans les objs dlg ?

    def load_shp_dlg_display(self):
        """Displays a dialog with a form getting information about :
            - the hill
            - the subhill
            - the id of the parcel
            - the shapefile containing the vertex
        """
        if (not self.load_shp_dlg) or \
                (not self.load_shp_dlg.isVisible()):
            self.load_shp_dlg = CIDLoadShpParcelDlg()
            self.load_shp_dlg.cancel_ok_btns.accepted.connect(
                self.load_and_display_shp)
            self.load_shp_dlg.cancel_ok_btns.rejected.connect(
                self.load_shp_dlg.close)
            self.load_shp_dlg.show()
            self.load_shp_dlg.exec_()

    def load_and_display_shp(self):
        """Loads and displays the vertex shapefile for edition"""
        self.load_shp_dlg.close()

        subhill = self.load_shp_dlg.subhill()
        parcel_number = self.load_shp_dlg.parcel_number()
        print(subhill, parcel_number)

        conn = get_connection()
        cur = conn.cursor()
        cur.execute(
            "\
insert into gis.parcels_to_valid (id_sub_hill_avenue, num_parcel) \
values (%s,%s) returning id",
            (self.load_shp_dlg.subhill(), self.load_shp_dlg.parcel_number()))
        id_parcel = cur.fetchone()[0]
        print("id_parcel", id_parcel)

        shp_layer = QgsVectorLayer(
            self.load_shp_dlg.shp_path(), "import_cid", "ogr")

        cur.execute("delete from gis.vertices_to_import")
        values = []
        for f in shp_layer.getFeatures():
            wkt = 'srid=32735;' + f.geometry().exportToWkt()
            values.append("('{}','{}'::geometry)".format(f['name'], wkt))
        query = "insert into gis.vertices_to_import (name, geom) values "
        query += ','.join(values)
        cur.execute(query)

        cur.execute("select gis.import_idc_vertices(%s)", (id_parcel,))

        conn.commit()

        l = self.layers.get("gis", "idc_parcels_to_valid")
        # update extent of affected layers
        if l:
            l.updateExtents()
            self.layers.get("gis", "idc_vertices_to_valid").updateExtents()
            self.layers.get("gis", "isolated_vertices").updateExtents()
            self.layers.get(
                "gis", "segments_idc_parcels_to_valid").updateExtents()
            # zoom to the new feature
            self.mapCanvas.zoomToFeatureIds(l, [id_parcel])

    def enable_disable_action_vertex(self, action_name):
        """
            Activate / desactivate the icon actions
            Returns a flat to know if the action is enable of disable
        """
        action = self.actions[action_name]
        new_active_action = True

        if self.active_action_name:
            active_action = self.actions[self.active_action_name]
            active_action.setChecked(False)

            if self.active_action_name in self.stop_action_function:
                # there exists a method to execute at the end of the action
                # so we have to execute this method
                self.stop_action_function[self.active_action_name]()

            if action_name == self.active_action_name:
                # no more action is active
                new_active_action = False
                self.active_action_name = None

        if new_active_action:
            self.active_action_name = action_name
            action.setChecked(True)

        return new_active_action

    def add_vertex(self):
        enable_action = self.enable_disable_action_vertex('add_vertex')

        if enable_action:
            add_vertex_layer = self.layers.get(
                'gis', 'segments_idc_parcels_to_valid')
            self.iface.setActiveLayer(add_vertex_layer)
            self.maptool = QgsMapToolEmitPoint(self.mapCanvas)
            self.maptool.canvasClicked.connect(self.add_vertex_at)
            self.mapCanvas.setMapTool(self.maptool)

    def add_vertex_at(self, new_vertex_point):
        add_vertex_layer = self.layers.get(
            'gis', 'segments_idc_parcels_to_valid')
        add_vertex_layer.startEditing()

        new_vertex_geom = QgsGeometry.fromPoint(new_vertex_point)
        new_vertex_buffer = new_vertex_geom.buffer(
            (self.mapCanvas.mapUnitsPerPixel() * 2), 0)
        new_vertex_rect = new_vertex_buffer.boundingBox()

        selected_edges = add_vertex_layer.getFeatures(new_vertex_rect)

        edge = QgsFeature()
        if selected_edges.nextFeature(edge):
            pt_wkt_txt = "'srid=32735;point({} {})'::geometry"\
                .format(new_vertex_point.x(), new_vertex_point.y())
            sql = 'SELECT gis.add_idc_vertex_to_parcel({},{},{},{})'\
                .format(
                    edge['id_parcel'], edge['id_vertex_before'],
                    edge['id_vertex_after'], pt_wkt_txt)
            transaction = add_vertex_layer.dataProvider().transaction()
            transaction.executeSql(sql)

            # FIXME est-ce que ca peut ê simplifié? (copier/coller action Hugo)
            for l in QgsProject.instance().mapLayers().values():
                if '"gis"."idc_vertices_to_valid"' in l.source():
                    # make_layer_dirty(l, 'id')
                    # def make_layer_dirty(layer, field_name):
                    layer = l
                    field_name = 'id'
                    feature = next(layer.getFeatures())
                    fake_value = feature.attribute(field_name)
                    fake_field_id = feature.fieldNameIndex(field_name)
                    # sets id to id just to dirty buffer
                    layer.editBuffer().changeAttributeValue(
                        feature.id(), fake_field_id, fake_value)
                    l.triggerRepaint()
            add_vertex_layer.triggerRepaint()
        else:
            print('NO selection')
        add_vertex_layer.commitChanges()

    def move_vertex(self):
        enable_action = self.enable_disable_action_vertex('move_vertex')

        if enable_action:
            vertex_layer = self.layers.get('gis', 'idc_vertices_to_valid')
            self.iface.setActiveLayer(vertex_layer)
            vertex_layer.startEditing()
            self.iface.actionMoveFeature().trigger()

    def commit_changes(self):
        vertex_layer = self.layers.get('gis', 'idc_vertices_to_valid')
        vertex_layer.commitChanges()
        self.pan_trigger()

    def delete_vertex(self):
        enable_action = self.enable_disable_action_vertex('delete_vertex')

        if enable_action:
            vertex_layer = self.layers.get('gis', 'idc_vertices_to_valid')
            self.iface.setActiveLayer(vertex_layer)

            self.maptool = QgsMapToolIdentifyFeature(
                self.mapCanvas, vertex_layer)
            self.maptool.featureIdentified.connect(
                self.delete_identified_vertex)
            self.mapCanvas.setMapTool(self.maptool)

    def delete_identified_vertex(self, feat):
        if(feat):
            vertex_layer = self.layers.get('gis', 'idc_vertices_to_valid')
            vertex_layer.startEditing()
            vertex_layer.deleteFeature(feat.id())
            vertex_layer.commitChanges()
            self.iface.mapCanvas().refresh()

    def add_ring(self, feat):
        """
        Trigger the addRing action on the layer idc_parcels_to_valid. This
        action allow the user to create ring into a polygon
        """
        enable_action = self.enable_disable_action_vertex('add_ring')

        if enable_action:
            parcels_layer = self.layers.get('gis', 'idc_parcels_to_valid')
            self.iface.setActiveLayer(parcels_layer)
            parcels_layer.startEditing()
            self.iface.actionAddRing().trigger()

    def reshape(self, feat):
        """
        Trigger the reshape / the cut feature action on the layer
        idc_parcels_to_valid. This action allow the user to cut external part
        of a polygone
        """
        enable_action = self.enable_disable_action_vertex('reshape')

        if enable_action:
            parcels_layer = self.layers.get('gis', 'idc_parcels_to_valid')
            self.iface.setActiveLayer(parcels_layer)
            parcels_layer.startEditing()
            reshapeAction = [
                o for o in self.iface.mainWindow().findChildren(QAction)
                if o.objectName() == 'mActionReshapeFeatures'][0]
            reshapeAction.trigger()

    def edit_inventory(self):
        enable_action = self.enable_disable_action_vertex('edit_inventory')
        if enable_action:
            parcels_layer = self.layers.get('gis', 'idc_parcels_to_valid')
            self.iface.setActiveLayer(parcels_layer)
            parcels_layer.startEditing()
            self.iface.actionIdentify().trigger()

    def print_inventory(self):
        enable_action = self.enable_disable_action_vertex('print_inventory')

        if enable_action:
            # FIXME tjs meme code a regrouper
            parcel_layer = self.layers.get(
                'gis', 'idc_parcels_to_valid')
            self.iface.setActiveLayer(parcel_layer)
            self.maptool = QgsMapToolEmitPoint(self.mapCanvas)
            self.maptool.canvasClicked.connect(self.print_inventory_at)
            self.mapCanvas.setMapTool(self.maptool)

    def print_inventory_at(self, point_clicked):
        # FIXME idem tjs meme code
        parcel_layer = self.layers.get(
            'gis', 'idc_parcels_to_valid')

        click_geom = QgsGeometry.fromPoint(point_clicked)
        click_buffer = click_geom.buffer(
            (self.mapCanvas.mapUnitsPerPixel() * 2), 0)
        click_rect = click_buffer.boundingBox()

        selected_parcels = parcel_layer.getFeatures(click_rect)

        parcel = QgsFeature()
        if selected_parcels.nextFeature(parcel):
            out_path = os.path.join(
                tempfile.gettempdir(),
                "inventory_{}.pdf".format(parcel['id']))

            pv_generator = CIDPvGenerator(self.plugin_dir, parcel)
            pv_generator.create_pdf_at(out_path)

            # ask the OS to open the generated PDF
            QDesktopServices.openUrl(QUrl.fromLocalFile(out_path))

    def validate_inventory(self):
        enable_action = self.enable_disable_action_vertex('validate_inventory')
        if enable_action:
            print('Validate inventory')

    def pan_trigger(self):
        self.iface.actionPan().trigger()
