from PyQt5.QtGui import QIcon, QDesktopServices
from PyQt5.QtXml import QDomDocument
from qgis.core import QgsComposition, QgsProject
from PyQt5.QtWidgets import QAction, QApplication, QMessageBox
import tempfile
import os.path
from PyQt5.QtCore import QUrl, Qt


class ComposerValidation(object):

    def __init__(
        self, feature, template_path, iface, substitution_map=dict(),
        zoom_to_feature=False
    ):
        self.feature = feature
        self.template_path = template_path
        self.iface = iface
        self.substitution_map = substitution_map
        self.zoom_to_feature = zoom_to_feature

    def export_img_path(self, ext):
        """
        ext: pdf ou png
        """
        return os.path.join(tempfile.gettempdir(), "export.{}".format(ext))

    def export_png(self):
        img = self.composition.printPageAsRaster(0)
        img_path = self.export_img_path('png')
        img.save(img_path)
        return img_path

    def export_pdf(self):
        # TODO
        pass

    def get_substitution_map(self):
        """
        Retourne un dict d tel que pour toute clef key, tout texte [key] sera
        remplace par d[key]

        Exemple si d = {'nom':'marc'}, alors le texte "Bonjour [nom]" sera
        remplace par "Bonjour marc".
        """
        return self.substitution_map

    def generate_composer(self):
        template = QDomDocument()
        with open(self.template_path, encoding="utf-8") as template_file:
            template.setContent(template_file.read())
        template_file.close()

        self.composition = QgsComposition(QgsProject.instance())
        self.composition.loadFromTemplate(
            template, self.get_substitution_map())

    def exec_(self):
        """
        Affichage du composer puis quand valider creation de l'image
        """
        self.generate_composer()

        self.composer_iface = self.iface.openComposer(self.composition)

        if self.zoom_to_feature:
            composer_map_item = self.composition.composerMapItems()[0] #FIXME only one map
            composer_map_item.zoomToExtent(self.feature.geometry().boundingBox())

        self.check_composer_window = self.composer_iface.parent()

        toolbar = self.check_composer_window.addToolBar('sif-td')
        icon = QIcon(':/plugins/PAGGF/img/icon_validate.png')
        action = QAction(
            icon, 'Valider le composer', self.check_composer_window)
        action.triggered.connect(self.exec_callback)
        action.setEnabled(True)
        toolbar.addAction(action)

    def exec_callback(self):
        QApplication.setOverrideCursor(Qt.WaitCursor)
        img_path = self.export_png()
        QApplication.restoreOverrideCursor()
        
        self.check_composer_window.close()

        QMessageBox.information(None, "Export réussi", "Export généré dans {}".format(img_path))
        QDesktopServices.openUrl(
            QUrl.fromLocalFile(img_path))
