from PyQt5.QtXml import QDomDocument
from qgis.core import QgsComposition, QgsProject, QgsComposerHtml, \
    QgsComposerFrame
import os.path
from ..paggf_layers import LoadedLayers


class BMPVGeneator(object):
    """Generate pdf file that contains the CID PV"""
    def __init__(self, plugin_dir, parcel):
        self.pv = None  # The pv that is generated
        self.parcel = parcel
        self.plugin_dir = plugin_dir
        self.layers = LoadedLayers()

    def create_pdf_at(self, out_path):
        """Generate the pdf at a given path"""
        template_path = os.path.join(
            self.plugin_dir, 'composer/bm_pv_registration.qpt')

        template_content = None
        with open(template_path, encoding="utf-8") as template_file:
            template_content = template_file.read()
        template_file.close()

        template_qdoc = QDomDocument()
        template_qdoc.setContent(template_content)

        self.pv = QgsComposition(QgsProject.instance())
        self.pv.loadFromTemplate(
            template_qdoc,
            substitutionMap={'QGIS_PLUGIN_DIR': self.plugin_dir})

        l = self.layers.get("gis", "vertices_angles_distances")
        print(self.parcel['id'])

        vertex_table_content = ''

        for v in l.getFeatures():
            if v['parcel_num_at_creation'] == self.parcel['id']:
                vertex_table_content += "\
<tr>\
    <td>{}{}-{}</td>\
    <td>{:.5f}</td>\
    <td>{:.5f}</td>\
    <td>{}</td>\
    <td>{}</td>\
</tr>".format(
                    ('P' if v['is_main'] else 'S'),
                    v['parcel_num_at_creation'],
                    v['vertex_num_at_creation'],
                    v['angle'],
                    v['distance'],
                    '?',  # FIXME TODO
                    '?'  # FIXME TODO
                    )

        vertex_table = "\
<style>\
table {\
    font-size: 10px;\
    border-collapse: collapse;\
}\
\
table, th, td {\
    border: 1px solid black;\
}\
</style>\
<table>\
    <thead>\
        <tr>\
            <th>Sommet du périmètre et leur description</th>\
            <th>Longueur des cotés réduite à l'horizon<br>(1)</th>\
            <th>Angles aux sommets<br><s>Grades</s>(2)<br>Degrés(2)<br>(1)</th>\
            <th colspan=\"2\">Autres renseignements éventuels permettant le \
                calcul de la superficie, l’établissement, l’orientation du \
                croquis (azimut ou gisement ou orientation d’un côté) ainsi \
                que le repérage des sommets</th>\
            <th>Description des côtés</th>\
            <th>Tenant et aboutissants</th>\
        </tr>\
    </thead>\
    <tbody>\
"

        vertex_table += vertex_table_content

        vertex_table += "\
    </tbody>\
</table>\
"

        # see http://programtalk.com/python-examples/qgis.core.QgsComposerFrame/
        # A4 Portrait
        # TODO: Will break when we try to use larger print layouts TS
        paper_width = 210
        paper_height = 297

        margin_left = 10
        margin_top = 20

        html_item = QgsComposerHtml(self.pv, False)

        html_frame = QgsComposerFrame(
            self.pv,
            html_item,
            margin_left,
            margin_top,
            paper_width - 2 * margin_left,
            paper_height - 2 * margin_top)

        html_frame.setItemPosition(
            margin_left,
            margin_top,
            page=2
        )

        html_item.addFrame(html_frame)



        # noinspection PyUnresolvedReferences
        html_item.setContentMode(QgsComposerHtml.ManualHtml)
        # noinspection PyUnresolvedReferences
        html_item.setResizeMode(QgsComposerHtml.RepeatUntilFinished)
        html_item.setHtml(vertex_table)
        # RMN: This line below breaks in InaSAFE Headless after one
        # successful call. This is because the function is not
        # thread safe. Can't do anything about this, so avoid calling this
        # function in multithreaded way.
        html_item.loadHtml()


        self.pv.setAtlasMode(QgsComposition.PreviewAtlas)
        self.pv.atlasComposition().prepareForFeature(self.parcel)
        self.pv.exportAsPDF(out_path)
