from .paggf_core import PAGGFCore, PAGGFAction
from . import paggf_version
from .ui.ALL_load_parcel import BMLoadParcelDlg, AllLoadParcelFileWidget, \
    AllLoadParcelManualWidget
from .composer.bm_pv_generator import BMPVGeneator
from .ui.welcome import WelcomeBMDlg


class PAGGF_BM(PAGGFCore):
    TITLE_GEOM = 'Géométrie parcelle BM'
    TITLE_ATTR = 'Attributs parcelle BM'
    TITLE_CROQUIS = 'Croquis orienté parcelle BM'
    TITLE_ANGLE = 'Ajout des angles et distances dans le PV bornage arpentage'
    TITLE_PV = 'PV parcelle BM'
    TITLE_PARAM = 'Paramétrage et maintenance BD BM'

    def __init__(self, iface):
        print('Init PAGGF BM')
        super(PAGGF_BM, self).__init__(
            iface, paggf_version.BM, 'BM et immatriculation TD')

        self.welcome_dlg_class = WelcomeBMDlg

        self.enable_paggf_action(
            'load_parcel_from_file', {
                'menu_title': 'Charger fichiers de points de sommets BM (shp, gpx ou csv)',
                'dlg_class': BMLoadParcelDlg,
                'load_widget_class': AllLoadParcelFileWidget,
                'tb_name': self.TITLE_GEOM
            })

        self.enable_paggf_action(
            'load_parcel_manually', {
                'menu_title': "Saisir manuellement les coordonnées XY des \
sommets d'une parcelle BM",
                'dlg_class': BMLoadParcelDlg,
                'load_widget_class': AllLoadParcelManualWidget,
                'tb_name': self.TITLE_GEOM
            })

        self.enable_paggf_action(
            'edit_data',
            {
                'menu_title': "Saisir manuellement le contenu d'un PV BM",
                'pg_schema_name': 'gis',
                'pg_table_name': 'parcels_to_valid',
                'tb_name': self.TITLE_ATTR,
            })

        self.enable_paggf_action(
            'print_pv',
            {
                'menu_title': 'Exportation et Impression PV bornage arpentage',
                'pg_schema_name': 'gis',
                'pg_table_name': 'parcels_to_valid',
                'pv_generator_cls': BMPVGeneator,
                'tb_name': self.TITLE_PV,
            }
        )

    def toolbar_list_def(self):
        return [
            (self.TITLE_GEOM, self.TITLE_GEOM),
            (self.TITLE_ATTR, self.TITLE_ATTR),
            (self.TITLE_CROQUIS, self.TITLE_CROQUIS),
            (self.TITLE_PV, self.TITLE_PV),
            (self.TITLE_ANGLE, self.TITLE_ANGLE),
            (self.TITLE_PARAM, self.TITLE_PARAM),
        ]

    def paggf_menu_content(self):
        return [  # DEFINITION DE L'ARBRE DU MENU
            # quand (nom, list) = sous menu
            # quand (nom, nom_action) = un access a une action
            (self.TITLE_GEOM, [
                PAGGFAction.actions['load_parcel_from_file'],  # Charger fichiers de points de sommets BM (shp, gpx ou csv)
                PAGGFAction.actions['load_parcel_manually'],  # Saisir manuellement les coordonnées XY des sommets d'une parcelle BM
                "Saisir manuellement les objets internes",
                "Editer et valider la géométrie d'une parcelle BM"
            ]),
            (self.TITLE_ATTR, [
                "Charger automatiquement les attributs PV BM",
                PAGGFAction.actions['edit_data']  # Saisir manuellement le contenu d'un PV BM
            ]),
            (self.TITLE_CROQUIS, [
                "Préparer et valider un croquis orienté de parcelle BM",
                "Exporter un croquis BM en pdf ou image"
            ]),
            (self.TITLE_ANGLE),
            PAGGFAction.actions['print_pv'],  # Exportation et Impression PV bornage arpentage
            (self.TITLE_PARAM, [
                "Paramétrage BD et plugin BM",
                "Synchronisation avec BD centrale"
            ])
        ]
