from .paggf_core import PAGGFCore, PAGGFAction
from . import paggf_version
from .ui.ALL_load_parcel import CIDLoadParcelDlg, AllLoadParcelFileWidget, \
    AllLoadParcelManualWidget
from .composer.cid_inventory_generator import CIDPvGenerator
from .ui.welcome import WelcomeCIDDlg
from qgis.core import QgsEditFormConfig
import os.path


class PAGGF_CID(PAGGFCore):
    TITLE_GEOM = 'Géométrie parcelle ID'
    TITLE_ATTR = 'Attributs parcelle ID'
    TITLE_CROQUIS = 'Croquis orienté parcelle ID'
    TITLE_PV = 'PV parcelle ID'
    TITLE_RAPP_COM = 'Rapport communal ID et autres vues/exports'
    TITLE_CONSULT = 'Consultation et requête inventaire TD communal'
    TITLE_PARAM = 'Paramétrage et maintenance BD ID'

    TITLE_TB_ARCHIVE = "Outils d'archivage"
    TITLE_TB_GEOM = "Outils d'édition de geom"

    def __init__(self, iface):
        super(PAGGF_CID, self).__init__(
            iface, paggf_version.CID, 'ID et inventaire TD')

        idc_parcels_layer = self.layers.get("gis", "idc_parcels_to_valid")
        edit_idc_parcel_form_conf = idc_parcels_layer.editFormConfig()
        edit_idc_parcel_form_conf = QgsEditFormConfig(
            edit_idc_parcel_form_conf)
        edit_idc_parcel_form_conf.setUiForm(
            os.path.join(self.plugin_dir, 'ui/CID_inventory_form.ui'))
        edit_idc_parcel_form_conf.setLayout(QgsEditFormConfig.UiFileLayout)
        edit_idc_parcel_form_conf.setInitCodeSource(
            QgsEditFormConfig.CodeSourceFile)
        edit_idc_parcel_form_conf.setInitFilePath(
            os.path.join(self.plugin_dir, 'ui/CID_inventory_form_init.py')
        )
        edit_idc_parcel_form_conf.setInitFunction('on_form_load')

        idc_parcels_layer.setEditFormConfig(edit_idc_parcel_form_conf)

        self.welcome_dlg_class = WelcomeCIDDlg
        self.enable_paggf_action(
            'load_parcel_from_file', {
                'menu_title': 'Charger fichiers de points de sommets ID \
(shp, gpx ou csv)',
                'dlg_class': CIDLoadParcelDlg,
                'load_widget_class': AllLoadParcelFileWidget,
                'tb_name': self.TITLE_GEOM
            })

        self.enable_paggf_action(
            'load_parcel_manually', {
                'menu_title': "Saisir manuellement les coordonnées XY des \
sommets d'une parcelle ID",
                'dlg_class': CIDLoadParcelDlg,
                'load_widget_class': AllLoadParcelManualWidget,
                'tb_name': self.TITLE_GEOM
            })

        self.enable_paggf_action(
            'edit_data',
            {
                'menu_title': "Saisir manuellement le contenu d'un PV CID",
                'pg_schema_name': 'gis',
                'pg_table_name': 'idc_parcels_to_valid',
                'tb_name': self.TITLE_ATTR
            })

        self.enable_paggf_action('add_vertex', {'tb_name': self.TITLE_TB_GEOM})
        self.enable_paggf_action('move_vertex', {'tb_name': self.TITLE_TB_GEOM})
        self.enable_paggf_action('delete_vertex', {'tb_name': self.TITLE_TB_GEOM})
        self.enable_paggf_action('add_ring', {'tb_name': self.TITLE_TB_GEOM})
        self.enable_paggf_action('reshape', {'tb_name': self.TITLE_TB_GEOM})

        self.enable_paggf_action('archive_parcel', {
            'pg_schema_name': 'gis',
            'pg_table_name': 'idc_parcels_to_valid',
            'tb_name': self.TITLE_TB_ARCHIVE,
        })

        self.enable_paggf_action('copy_parcel', {
            'pg_schema_name': 'gis',
            'pg_table_name': 'current_idc_parcels',
            'tb_name': self.TITLE_TB_ARCHIVE,
        })

        self.enable_paggf_action('pv_upload_signatures', {
            'menu_title': 'Fusion PV ID et scan des signatures',
            'pg_schema_name': 'gis',
            'pg_table_name': 'idc_parcels_to_valid',
            'pv_generator_cls': CIDPvGenerator,
            'tb_name': self.TITLE_PV,
        })

        self.enable_paggf_action('pv_w_signatures_print', {
            'menu_title': 'Génération et impression PV ID',
            'pg_schema_name': 'gis',
            'pg_table_name': 'idc_parcels_to_valid',
            'pv_generator_cls': CIDPvGenerator,
            'tb_name': self.TITLE_PV,
        })

        self.enable_paggf_action('load_attributes_from_file', {
            'pg_schema_name': 'gis',
            'pg_table_name': 'idc_parcels_to_valid',
            'menu_title': 'Charger automatiquement les attributs PV CID',
            'tb_name': self.TITLE_ATTR,
        })

        self.enable_paggf_action(
            'composer_validation',
            {
                'menu_title': "Préparer et valider un croquis orienté de parcelle ID",
                'pg_schema_name': 'gis',
                'pg_table_name': 'idc_parcels_to_valid',
                "tb_name": self.TITLE_CROQUIS,
            }
        )

        self.enable_paggf_action(
            'composer_map_munip_inventory',
            {
                'menu_title': "Edition et impression de la carte communale",
                "tb_name": self.TITLE_RAPP_COM,
            }
        )

    def toolbar_list_def(self):
        return [
            (self.TITLE_GEOM, self.TITLE_GEOM),
            (self.TITLE_ATTR, self.TITLE_ATTR),
            (self.TITLE_CROQUIS, self.TITLE_CROQUIS),
            (self.TITLE_PV, self.TITLE_PV),
            (self.TITLE_RAPP_COM, self.TITLE_RAPP_COM),
            (self.TITLE_CONSULT, self.TITLE_CONSULT),
            (self.TITLE_PARAM, self.TITLE_PARAM),
            (self.TITLE_TB_ARCHIVE, self.TITLE_TB_ARCHIVE),
            (self.TITLE_TB_GEOM, self.TITLE_TB_GEOM),
        ]

    def paggf_menu_content(self):
        return [  # DEFINITION DE L'ARBRE DU MENU
            # quand (nom, list) = sous menu
            # quand (nom, nom_action) = un access a une action
            (self.TITLE_GEOM, [
                PAGGFAction.actions['load_parcel_from_file'],  # Charger fichiers
                # de points de sommets ID (shp, gpx ou csv)
                PAGGFAction.actions['load_parcel_manually'],  # Saisir manuellement
                # les coordonnées XY des sommets d'une parcelle ID",
                "Editer et valider la géométrie d'une parcelle ID"
            ]),
            (self.TITLE_ATTR, [
                PAGGFAction.actions['load_attributes_from_file'],
                PAGGFAction.actions['edit_data']
            ]),
            (self.TITLE_CROQUIS, [
                PAGGFAction.actions['composer_validation'], # Préparer et valider
                # un croquis orienté de parcelle ID
                "Exporter un croquis ID en pdf ou image"
            ]),
            (self.TITLE_PV, [
                PAGGFAction.actions['pv_w_signatures_print'],  # Génération et
                # impression PV ID
                PAGGFAction.actions['pv_upload_signatures'],  # Fusion PV ID et scan
                # des signatures
            ]),
            (self.TITLE_RAPP_COM, [
                "Edition et impression de l'affichage restitution provisoire",
                PAGGFAction.actions['composer_map_munip_inventory'],  # Edition et impression de la carte communale
                "Exporter le plan global des TDs identifiées par colline",
                "Préparer et exporter/imprimer un extrait de croquis de \
parcelle ID"
            ]),
            (self.TITLE_CONSULT, [
                "Visualisation synthétique de la BD communale",
                "Requête par attribut"
            ]),
            (self.TITLE_PARAM, [
                "Paramétrage BD et plugin ID",
                "Synchronisation avec BD centrale"
            ])
        ]
