from PyQt5.QtWidgets import QLabel, QLineEdit, QFormLayout, QComboBox, \
    QDialogButtonBox, QListWidget, QPushButton, QWidget, QDialog
from PAGGF.paggf_layers import LoadedLayers
from PAGGF.db import get_connection
from qgis.core import QgsExpression, QgsFeatureRequest, QgsFeature
from PAGGF.ui.form_helper import fill_combobox_with_layer_content, \
    QListWidgetEditor, get_widget_dict

layers = LoadedLayers()

def on_form_load(dialog, layer, feature):
    """
    Function executed when inventory form is loaded
    """
    if feature.attributes() == []:
        # form loaded without data, bypass
        return

    bottom_form = dialog.findChild(QFormLayout, 'bottom_form')

    origin_other = dialog.findChild(QLineEdit, 'origin_other')
    origin_other_label = dialog.findChild(QLabel, 'origin_other_label')
    ownership_origin = dialog.findChild(QComboBox, 'ownership_origin')

    manager_other = dialog.findChild(QLineEdit, 'manager_other')
    manager_other_label = dialog.findChild(QLabel, 'manager_other_label')
    manager = dialog.findChild(QComboBox, 'manager')

    usage_other = dialog.findChild(QLineEdit, 'usage_other')
    usage_other_label = dialog.findChild(QLabel, 'usage_other_label')
    usage = dialog.findChild(QComboBox, 'usage')

    assignment_other = dialog.findChild(QLineEdit, 'assignment_other')
    assignment_other_label = dialog.findChild(QLabel, 'assignment_other_label')
    assignment = dialog.findChild(QComboBox, 'assignment')

    def create_slot_for_field(
            field, value_other, field_other, field_other_label):
        def value_changed_slot():
            if field.currentData() == value_other:
                field_other.show()
                field_other_label.show()
            else:
                field_other.hide()
                field_other_label.hide()
        return value_changed_slot

    for field, value_other, field_other, field_other_label in [
        (ownership_origin, 'Other', origin_other, origin_other_label),
        (usage, 'Other', usage_other, usage_other_label),
        (assignment, 'Other', assignment_other, assignment_other_label)
    ]:
        field_other.hide()
        field_other_label.hide()

        value_changed_slot = create_slot_for_field(
            field, value_other, field_other, field_other_label)
        field.currentTextChanged.connect(value_changed_slot)

    adjacent_people_tab_init(dialog, feature)
    commission_member_tab_init(dialog, feature)
    crc_commission_member_tab_init(dialog, feature)
    witness_tab_init(dialog, feature)


def adjacent_people_tab_init(dialog, inventory):
    """
    Initialize the user interaction for the tad 'Adjacent-people'
    """
    global adjacent_people_list_editor

    inventory_id = inventory['id']

    widget = get_widget_dict(dialog, [
        ('adjacent_people_list', QListWidget),
        ('adjacent_people_firstname', QLineEdit),
        ('adjacent_people_lastname', QLineEdit),
        ('adjacent_people_add', QPushButton),
        ('adjacent_people_delete', QPushButton)])

    adjacent_people_layer = layers.get('idc', 'inventories_adjacent_people')
    filter_exp = QgsExpression('"id_inventory" = {}'.format(inventory['id']))
    filter_req = QgsFeatureRequest(filter_exp)

    def get_new_data():
        ret = (widget['adjacent_people_lastname'].text(), widget['adjacent_people_firstname'].text())
        widget['adjacent_people_lastname'].setText('')
        widget['adjacent_people_firstname'].setText('')
        return ret

    def get_data_repr(data):
        return "{} {}".format(data[0],data[1])

    def get_init_data_list():
        ret = []
        filter_exp = QgsExpression('"id_inventory" = {}'.format(inventory['id']))
        filter_req = QgsFeatureRequest(filter_exp)
        layer = layers.get('idc', 'inventories_adjacent_people')

        for feature in layer.getFeatures(filter_req):
            ret.append((feature['lastname'], feature['firstname']))
        return ret

    def save_data_fct(data_list):
        conn = get_connection()
        cur = conn.cursor()
        row_people = "row("+"),row(".join(["'{}','{}'".format(d[1],d[0]) for d in data_list])+")"
        cur.execute("\
SELECT gis.inventory_set_adjacent_people({}, array[{}]::gis._adjacent_people_param[])".format(
            inventory_id, row_people))
        conn.commit()

    adjacent_people_list_editor = QListWidgetEditor(
        widget['adjacent_people_list'],
        widget['adjacent_people_add'],
        widget['adjacent_people_delete'],
        get_new_data,
        get_data_repr,
        get_init_data_list,
        save_data_fct)

def member_with_quality_tab_init(
        dialog, inventory,
        list_widget_name, firstname_field_name, lastname_field_name,
        add_button_name, quality_field_name,
        delete_button_name, db_table_name,
        quality_field_is_lineedit = True, # otherwise it is a combobox
        quality_db_table_name=None,
        db_function_name=None):
    inventory_id = inventory['id']

    if quality_field_is_lineedit:
        quality_field_type = QLineEdit
    else:
        quality_field_type = QComboBox

    widget = get_widget_dict(dialog, [
        (list_widget_name, QListWidget),
        (firstname_field_name, QLineEdit),
        (lastname_field_name, QLineEdit),
        (add_button_name, QPushButton),
        (delete_button_name, QPushButton),
        (quality_field_name, quality_field_type)])

    if quality_db_table_name:
        fill_combobox_with_layer_content(
            widget[quality_field_name],
            layers.get('idc', quality_db_table_name),
            'name',
            'id'
        )

    def get_new_data():
        if quality_field_is_lineedit:
            ret = (
                widget[lastname_field_name].text(),
                widget[firstname_field_name].text(),
                widget[quality_field_name].text(),
                widget[quality_field_name].text())
            widget[quality_field_name].setText('')
        else:
            ret = (
                widget[lastname_field_name].text(),
                widget[firstname_field_name].text(),
                widget[quality_field_name].currentText(),
                widget[quality_field_name].currentData())
        widget[lastname_field_name].setText('')
        widget[firstname_field_name].setText('')
        return ret

    def get_data_repr(data):
        return "{} {} ({})".format(data[0], data[1], data[2])

    def get_init_data_list():
        ret = []
        filter_exp = QgsExpression('"id_inventory" = {}'.format(inventory_id))
        filter_req = QgsFeatureRequest(filter_exp)
        layer = layers.get('idc', db_table_name)

        for feature in layer.getFeatures(filter_req):
            if quality_db_table_name:
                ret.append((feature['lastname'], feature['firstname'], feature['member_quality_id'], feature['member_quality_id']))
            else:
                ret.append((feature['lastname'], feature['firstname'], feature['quality'], feature['quality']))
        return ret

    def save_data_fct(data_list):
        conn = get_connection()
        cur = conn.cursor()
        row_people = "row("+"),row(".join(["'{}','{}','{}'".format(d[0],d[1],d[3]) for d in data_list])+")"
        cur.execute("SELECT gis.{}({}, array[{}]::gis._{}_param[])".format(
            db_function_name, inventory_id, row_people, db_function_name))
        conn.commit()

    return QListWidgetEditor(
        widget[list_widget_name],
        widget[add_button_name],
        widget[delete_button_name],
        get_new_data,
        get_data_repr,
        get_init_data_list,
        save_data_fct)

def commission_member_tab_init(dialog, inventory):
    global commission_members_list
    commission_members_list = member_with_quality_tab_init(
        dialog, inventory,
        'commission_members_list', 'commission_members_firstname',
        'commission_members_lastname', 'commission_members_add',
        'commission_members_quality', 'commission_members_delete',
        'commission_members', quality_field_is_lineedit=False,
        quality_db_table_name='commission_member_enum',
        db_function_name='inventory_set_commission_members')

def crc_commission_member_tab_init(dialog, inventory):
    global crc_commission_members_list
    crc_commission_members_list = member_with_quality_tab_init(
        dialog, inventory,
        'crc_commission_members_list', 'crc_commission_members_firstname',
        'crc_commission_members_lastname', 'crc_commission_members_add',
        'crc_commission_members_quality', 'crc_commission_members_delete',
        'crc_commission_members', quality_field_is_lineedit=False,
        quality_db_table_name='crc_commission_member_enum',
        db_function_name='inventory_set_crc_commission_members')

def witness_tab_init(dialog, inventory):
    global witness_list
    witness_list = member_with_quality_tab_init(
        dialog, inventory,
        'witness_list', 'witness_firstname',
        'witness_lastname', 'witness_add',
        'witness_quality', 'witness_delete',
        'witnesses',
        db_function_name='inventory_set_witnesses')
