# -*- coding: utf-8 -*-
"""
/***************************************************************************
 CIDLoadShpParcelDlg
                                 A QGIS plugin
 -
                             -------------------
        begin                : 2017-06-20
        git sha              : $Format:%H$
        copyright            : (C) 2017 by PAGGF
        email                : info@paggf.bi
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

import os

from PyQt5 import uic
from PyQt5.QtWidgets import QDialog
from qgis.core import QgsExpressionContextUtils, QgsProject

from ..db import get_connection
from ..paggf_layers import LoadedLayers
from .dlg_with_shp_loader import DlgWithShpLoader

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'CID_load_shp_parcel.ui'))


class CIDLoadShpParcelDlg(QDialog, FORM_CLASS, DlgWithShpLoader):
    def __init__(self, parent=None):
        """Constructor.

        This dialog has to know the id of municipality on which the user
        works. This id is stored in the variable 'municipality_id'.
        """
        QDialog.__init__(self, parent)
        # Set up the user interface from Designer.
        self.setupUi(self)
        DlgWithShpLoader.__init__(self)

        self.project = QgsProject.instance()
        self.layers = LoadedLayers()
        self.municipality_id = QgsExpressionContextUtils \
            .projectScope(self.project).variable('id_municipality')
        self.municipality_name = None

        municipality_layer = self.layers.get("ref", "municipalities")
        if municipality_layer:
            for f in municipality_layer.getFeatures():
                if f['id'] == self.municipality_id:
                    self.municipality_name = f['name']

        if self.municipality_name:
            self.municipality_field.setText(self.municipality_name)
        else:
            self.municipality_field.setText('Inconnu')

        hill_layer = self.layers.get("ref", "hill_districts")
        if hill_layer:
            for f in hill_layer.getFeatures():
                if f['id'].startswith(self.municipality_id):
                    self.hill_field.addItem(
                        "{} - {}".format(f['id'], f['name']), f['id'])

        self.subhills = {}
        subhill_layer = self.layers.get("ref", "sub_hill_avenues")
        if subhill_layer:
            for f in subhill_layer.getFeatures():
                self.subhills[f['id']] = f['name']

        self.hill_field.currentIndexChanged[int].connect(
            self.when_hill_selection_changed)

        self.when_hill_selection_changed(0)

    def when_hill_selection_changed(self, index):
        """
        Called when the user change the selection of the hill (the selected
        hill in the combo-box hill_field has changed)

        Params
        ------
        int index : new index in the combobox of the selected hill
        """
        hill_id = self.hill_field.itemData(index)
        self.subhill_field.clear()
        for id_, name in self.subhills.items():
            if id_.startswith(hill_id):
                self.subhill_field.addItem("{} - {}".format(id_, name), id_)

        # mise à jour numéro parcelle
        conn = get_connection()
        cur = conn.cursor()
        cur.execute("select gis.next_parcel_num('{}')".format(hill_id))
        parcel_number = cur.fetchone()[0]
        self.parcel_number_field.setText(str(parcel_number))

    def subhill(self):
        return int(self.subhill_field.currentData())

    def parcel_number(self):
        return int(self.parcel_number_field.text())
