from PyQt5.QtWidgets import QDialog
"""
Function and object for interacting with forms :
    fct:
        - fill_combobox_with_layer_content
        - get_widget_dict()

    objects:
        - QListWidgetEditor
"""


def fill_combobox_with_layer_content(combobox, layer, field_text, field_value):
    """
    field_text -> the field to display in the combobox (visible by user)
    field_value -> the value to get for identifying the element selected
    """
    combobox.clear()
    for feature in layer.getFeatures():
        combobox.addItem(feature[field_text], feature[field_value])


def get_widget_dict(dialog, widget_desc):
    widget_dict = dict()

    for widget_name, widget_type in widget_desc:
        widget_dict[widget_name] = dialog.findChild(widget_type, widget_name)

    return widget_dict


class QListWidgetEditor(object):
    def __init__(
            self, list_widget, button_widget_add, button_widget_delete,
            # button_box_ok,
            get_new_data, get_data_repr, get_init_data_list, save_data_fct):
        self.data_list = get_init_data_list()  # List with all the data
        self.data_list_edited = False

        self.list_widget = list_widget
        self.button_widget_add = button_widget_add
        self.button_widget_delete = button_widget_delete
        self.get_data_repr = get_data_repr # function that display data in list widget
        self.get_new_data = get_new_data
        self.save_data_fct = save_data_fct

        self.button_widget_add.clicked.connect(self.slot_add_elem)
        self.button_widget_delete.clicked.connect(self.slot_delete_elem)

        dlg = list_widget.window()
        if isinstance(dlg, QDialog):
            dlg.accepted.connect(self.slot_data_save)

        self.update_list()

    def update_list(self):
        self.list_widget.clear()

        for d in self.data_list:
            self.list_widget.addItem(self.get_data_repr(d))

    def slot_data_save(self):
        if self.data_list_edited:
            self.save_data_fct(self.data_list)

    def slot_add_elem(self):
        self.data_list_edited = True
        self.data_list.append(self.get_new_data())
        self.update_list()

    def slot_delete_elem(self):
        self.data_list_edited = True
        del self.data_list[self.list_widget.currentRow()]
        self.update_list()
